@@ -48,4 +48,11 @@ module Puppet::Indirector
   module ClassMethods
     attr_reader :indirection
   end
+
+
+  # Helper definition for indirections that handle filenames.
+  BadNameRegexp = Regexp.union(/^\.\./,
+                               %r{[\\/]},
+                               "\0",
+                               /(?i)^[a-z]:/)
 end
