@@ -23,7 +23,7 @@ class Net::HTTP
 
   # JJM: This is a "backport" of sorts to older ruby versions which
   # do not have this accessor.  See #896 for more information.
-  attr_accessor :enable_post_connection_check unless Net::HTTP.instance_methods.include? "enable_post_connection_check"
+  attr_accessor :enable_post_connection_check unless Net::HTTP.method_defined? "enable_post_connection_check"
 end
 
 # The base class for all of the clients.  Many clients just directly
@@ -82,11 +82,6 @@ class Puppet::Network::Client
 
       self.read_cert
 
-      # We have to start the HTTP connection manually before we start
-      # sending it requests or keep-alive won't work.  Note that with #1010,
-      # we don't currently actually want keep-alive.
-      @driver.start if @driver.respond_to? :start and Puppet::Network::HttpPool.keep_alive?
-
       @local = false
     elsif hash.include?(driverparam)
       @driver = hash[driverparam]
