@@ -103,6 +103,23 @@ class Puppet::Rails::Schema
           t.column :created_at, :datetime
         end
         add_index :param_names, :name
+
+        create_table :inventory_nodes do |t|
+          t.column :name, :string, :null => false
+          t.column :timestamp, :datetime, :null => false
+          t.column :updated_at, :datetime
+          t.column :created_at, :datetime
+        end
+
+        add_index :inventory_nodes, :name, :unique => true
+
+        create_table :inventory_facts, :id => false do |t|
+          t.column :node_id, :integer, :null => false
+          t.column :name, :string, :null => false
+          t.column :value, :text, :null => false
+        end
+
+        add_index :inventory_facts, [:node_id, :name], :unique => true
       end
     end
   ensure
